unit profile_unit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils,Graphics,
  TAGraph, TATools, TASeries,TATypes;

{Copyright (C) 2017 Frank Ranis

Sie haben das unentgeltliche Recht diese Software
(sowie den bereitgestellten Quelltext) für nicht gewerbliche Zwecke zu nutzen.
Unter der Voraussetzung das sie folgendes anerkennen:
Sie erkennen an , dass der Autor keinerlei Gewährleistung , Verpflichtungen oder
Haftung für Schäden , die durch die Nutzung der Software oder daraus gewonnener
Daten entstehen , übernimmt.

Es wird gewünscht , das geänderte Software (incl. Quelltext) wieder allen
interessierten Benutzern zur Verfügung gestellt wird.
Dabei bitte die Versions-Nummer mit Datum und Namen des neuen Autors anpassen ,
sowie Hinweise zu den Änderungen im Quelltext eintragen .
Das geänderte Programmpaket bitte wieder (gezippt) im RC-Network-WIKI
(Programmname_Vers_xxx.zip) zusätzlich hochladen, alte Versionen stehen lassen.
}

// Version mit Datum , Autor , Änderungen
// Vers. 1.0 , 21.11.2017 , Frank Ranis
// Erstellt mit Lazarus V1.6.4 / Freepascal V3.0.2
// Import: (Selig.dat) , (Konopka.dat) , (Wiechers.koo)
// Export: (Selig.dat) , (Konopka.dat) , (Wiechers.koo)

type

 // Koordinaten-Punkt-Object
 Tkoordi=object
  x,y,z:double;
 end;

 // Profil-Object
 TProf=object
  Prof_Titel:string; // Titelzeile (Erste Zeile in der Dat-Datei)
  Prof_Dateiname:string; // Datei-Filename , ohne Pfad
  koordinaten:array of Tkoordi; // Array für die X/Y-Koordinaten als Double
  function Getcount:integer; // holt die Anzahl der Koordinatenpaare
  procedure clear; // setzt das Koordinatenarray zurück auf 0
  procedure send_to_Chart(chart:tchart); // überträgt die Profilkoordinaten an das Chart

  procedure load_selig_dat_from_file(filename: string);
  procedure load_konopka_dat_from_file(filename:string);
  procedure load_wiechers_koo_from_file(filename:string);

  procedure save_konopka_dat_to_file(filename:string);
  procedure save_selig_dat_to_file(filename:string);
  procedure save_wiechert_koo_to_file(filename:string);

  property count:integer read Getcount; // Anzahl der Koordinatenpaare holen;
 end;

implementation

{ TProf }

// holt die Anzahl der Koordinatenpaare
function TProf.Getcount:integer;
begin
 result:=length(koordinaten);
end;

// alte Inhalte löschen
procedure TProf.clear;
begin
 self.Prof_Dateiname:='';
 self.Prof_Titel:='';
 setlength(self.koordinaten,0);
end;

// läde eine Wiechers-Koo-Datei
// Fängt hinten oben an
// -> entlang der Oberseite zur Nase
// -> entlang der Unterseite zurück zur Endleiste unten
// In der ersten Zeile steht der Profilname und durch ',' getrennt
// die Anzahl der Koordinaten-Punkte
// die Normierung im File ist 100 und wird hier intern auf 1 skaliert
// als Koordinatenseparator dient ein ' '
// Der Dezimalseparator ist ein '.'
procedure TProf.load_wiechers_koo_from_file(filename: string);
var stringlist:Tstringlist;
    j:integer; // Zeilenzähler
    anz:integer; // Anzahl Koordinaten
    zeile:string;
    stringarray:Tstringarray; // eine zerlegte Koordinatenzeile
begin
 // Den Dateinamen merken
 self.Prof_dateiname:=extractfilename(filename);
 // Eine Stringliste anlegen , dient als Zwischenspeicher zum Laden des DAT-Files
 stringlist:=tstringlist.Create;
 // Das Dat-File (ist eine Textdatei) in die Stringlist laden
 stringlist.LoadFromFile(filename);

 // Wir nehmen die erste Zeile und zerlegen diese in Titel und Anzahl Koordinaten
 zeile:=stringlist[0];
 stringarray:=zeile.split([' , '],TStringSplitOptions.ExcludeEmpty); // String Split Helper von Lazarus
 // Profil Titel
 self.Prof_Titel:=stringarray[0];
 // Anzahl der Koordinaten
 anz:=strtoint(stringarray[1]);

 // Zeilen durchjuckeln und Werte rausziehen
 j:=0;
 repeat
   zeile:=stringlist[j+1];
   // Für Fließkommaoperationen Strtofloat wird der System-DecimalSeparator
   // benutzt , in der Oginal Datei stehen '.' als Dezimaltrenner.
   // Dies wird nun an das System angepasst fals z.B. im System ',' benutzt wird.
   zeile:=zeile.Replace('.',DefaultFormatSettings.DecimalSeparator); // String Replace Helper von Lazarus
   // Die Zeile in der ja X und Y aufgeführt sind , zerlegen und die getrennten
   // Teile in stringarray packen
   // In Stringarray[0] steht dann X und in Stringarray[1] Y drinn
   stringarray:=zeile.split([' '],TStringSplitOptions.ExcludeEmpty); // String Split Helper von Lazarus

   // Nun das Koordinatenarray um eine Stelle erweitern und X/Y als Double
   // eintragen
   // Konopka benutzt als Normierung 100 , wir skalieren diese nun auf 1
   // also alles durch 100 Teilen
   if (stringarray<>nil) and (stringarray[0]>'') and (stringarray[1]>'')
    then
     begin
      setlength(koordinaten,j+1);
      koordinaten[j].x:=strtofloat(Stringarray[0])/100;
      koordinaten[j].y:=strtofloat(Stringarray[1])/100;
     end;
   inc(j);
 Until j>anz;

 // Stringlist wird nicht mehr benötigt , also killen (Speicher freigeben)
 stringlist.Free;
end;

// läde eine Konopka-DAT-Datei
// Fängt hinten oben an
// -> entlang der Oberseite zur Nase
// -> entlang der Unterseite zurück zur Endleiste unten
// die Koordinaten fangen gleich in der ersten Zeile an
// als Koordinatenseparator dient ein ','
// die Normierung im File ist 100 und wird hier intern auf 1 skaliert
procedure TProf.load_konopka_dat_from_file(filename:string);
var stringlist:Tstringlist;
    j:integer; // Zeilenzähler
    zeile:string;
    stringarray:Tstringarray; // eine zerlegte Koordinatenzeile
begin
 // Den Dateinamen merken
 self.Prof_dateiname:=extractfilename(filename);
 // Eine Stringliste anlegen , dient als Zwischenspeicher zum Laden des DAT-Files
 stringlist:=tstringlist.Create;
 // Das Dat-File (ist eine Textdatei) in die Stringlist laden
 stringlist.LoadFromFile(filename);

 // Die Titelzeile rausziehen , geht nicht , da nicht vorhanden
 // wir nehmen einfach den Dateinamen
 // und machen den Anhang weg
 self.Prof_Titel:=copy(self.Prof_dateiname,1,pos('.',self.Prof_dateiname)-1);

 // jetzt die Koordinatenzeilen Stück für Stück in X Y zerlegen und in das
 // Koordinaten-Array (koordi_double) packen.
 j:=0;
 Repeat
  zeile:=stringlist[j];
  // Wir ersetzen den Koordinaten-Separator ',' durch Leezeichen
  zeile:=zeile.Replace(',',' '); // String Replace Helper von Lazarus
  // Für Fließkommaoperationen Strtofloat wird der System-DecimalSeparator
  // benutzt , in der Oginal Datei stehen '.' als Dezimaltrenner.
  // Dies wird nun an das System angepasst fals z.B. im System ',' benutzt wird.
  zeile:=zeile.Replace('.',DefaultFormatSettings.DecimalSeparator); // String Replace Helper von Lazarus
  // Die Zeile in der ja X und Y aufgeführt sind , zerlegen und die getrennten
  // Teile in stringarray packen
  // In Stringarray[0] steht dann X und in Stringarray[1] Y drinn
  stringarray:=zeile.split([' '],TStringSplitOptions.ExcludeEmpty); // String Split Helper von Lazarus

  // Nun das Koordinatenarray um eine Stelle erweitern und X/Y als Double
  // eintragen
  // Konopka benutzt als Normierung 100 , wir skalieren diese nun auf 1
  // also alles durch 100 Teilen
  if (stringarray<>nil) and (stringarray[0]>'') and (stringarray[1]>'')
   then
    begin
     setlength(koordinaten,j+1);
     koordinaten[j].x:=strtofloat(Stringarray[0])/100;
     koordinaten[j].y:=strtofloat(Stringarray[1])/100;
    end;
  inc(j);
 until j>stringlist.Count-1; // Ende der Datei

 // Stringlist wird nicht mehr benötigt , also killen (Speicher freigeben)
 stringlist.Free;
end;

// läde eine Selig-DAT-Datei
// Fängt hinten oben an
// -> entlang der Oberseite zur Nase
// -> entlang der Unterseite zurück zur Endleiste unten
// als Koordinatenseparator dient ein ' '
// Der Dezimalseparator ist ein '.'
procedure TProf.load_selig_dat_from_file(filename: string);
var stringlist:Tstringlist;
    j:integer; // Zeilenzähler
    zeile:string;
    stringarray:Tstringarray; // eine zerlegte Koordinatenzeile
begin
 // Den Dateinamen merken
 self.Prof_dateiname:=extractfilename(filename);
 // Eine Stringliste anlegen , dient als Zwischenspeicher zum Laden des DAT-Files
 stringlist:=tstringlist.Create;
 // Das Dat-File (ist eine Textdatei) in die Stringlist laden
 stringlist.LoadFromFile(filename);

 // Die Titelzeile rausziehen , hier steht der Beschreibungstext zum Profil
 self.Prof_Titel:=stringlist[0];

 // Die erste Zeile der Stringlist (Titeltext) ist für das Zerlegen der
 // Koordinaten störend , deshalb löschen wir diese weg
 stringlist.Delete(0);

 // jetzt die Koordinatenzeilen Stück für Stück in X Y zerlegen und in das
 // Koordinaten-Array (koordi_double) packen.
 j:=0;
 Repeat
  zeile:=stringlist[j];
  // Es kann vorkommen , das in einer Dat-Datei anstelle eines Leerzeichens zum
  // trennen von X Y ein Tabulator verwendet wird.
  // Wir ersetzen also eventuelle Tab's durch Leerzeichen
  // alle TAB's in Leezeichen umwandeln
  zeile:=zeile.Replace(#9,' '); // String Replace Helper von Lazarus
  // Für Fließkommaoperationen Strtofloat wird der System-DecimalSeparator
  // benutzt , in der Oginal Selig-Dat-Datei stehen '.' als Dezimaltrenner.
  // Dies wird nun an das System angepasst fals z.B. im System ',' benutzt wird.
  zeile:=zeile.Replace('.',DefaultFormatSettings.DecimalSeparator); // String Replace Helper von Lazarus
  // Die Zeile in der ja X und Y aufgeführt sind , zerlegen und die getrennten
  // Teile in stringarray packen
  // In Stringarray[0] steht dann X und in Stringarray[1] Y drinn
  stringarray:=zeile.split([' '],TStringSplitOptions.ExcludeEmpty); // String Split Helper von Lazarus

  // Nun das Koordinatenarray um eine Stelle erweitern und X/Y als Double
  // eintragen
  if (stringarray<>nil) and (stringarray[0]>'') and (stringarray[1]>'')
   then
    begin
     setlength(koordinaten,j+1);
     koordinaten[j].x:=strtofloat(Stringarray[0]);
     koordinaten[j].y:=strtofloat(Stringarray[1]);
    end;
  inc(j);
 until j>stringlist.Count-1; // Ende der Datei

 // Stringlist wird nicht mehr benötigt , also killen (Speicher freigeben)
 stringlist.Free;
end;

// das Profil im Konopka-Dat-Format speichern
procedure TProf.save_konopka_dat_to_file(filename: string);
var stringlist:Tstringlist;
    j:integer; // Zeilenzähler
    zeile:string;
    ext:string;
begin
 // eine Stringlist als Zwischenspeicher
 stringlist:=Tstringlist.Create;
 // In Konopka-Dat wird kein Titeltext verwendet , die Koordinaten werden
 // ab der ersten Zeile eingetragen
 for j:=low(self.koordinaten) to high(self.koordinaten) do
  begin
   // eine Zeile mit X- und Y-Koordinate für einen Punkt zusammenbauen
   // Konopka benutz eine Koordinatennormierung von 100 , also
   // alle Werte *100 nehmen
   zeile:=FormatFloat('0.00000',self.koordinaten[j].x*100)+' '+
          FormatFloat('0.00000',self.koordinaten[j].y*100);
   // je nach Spracheinstellung wird als Dzimalseparator ein '.' oder ','
   // verwendet
   // das Konopka-Format muß einen '.' als Dezimaltrenner haben
   // also eventuelle ',' durch einen '.' ersetzen
   zeile:=zeile.Replace(',','.');
   // als Koordinatenseparator wird im Konopka ein ',' verwendet
   // also ersetzen wird die Leerzeichen durch ','
   zeile:=zeile.Replace(' ',',');
   // Zeile an Stringlist anfügen
   stringlist.Add(zeile);
  end;

 // nachschauen , ob bei filename eine passende Dateierweiterung vorhanden ist
 ext:=uppercase(extractfileext(filename));
 if ext<>'.DAT'
  then
   begin
    // wenn ext leer
    if ext=''
     then filename:=filename+'.DAT'
     // wenn was anderes als '.DAT'drinn steht
     else
      begin
       filename:=copy(filename,1,pos('.',filename)-1);
       filename:=filename+'.DAT';
      end;
   end;

 // Stringlist abspeichern
 stringlist.SaveToFile(filename);

 // Stringlist wird nicht mehr benötigt , also killen (Speicher freigeben)
 stringlist.Free;
end;

// das Profil im Selig-Dat-Format speichern
procedure TProf.save_selig_dat_to_file(filename: string);
var stringlist:Tstringlist;
    j:integer; // Zeilenzähler
    zeile:string;
    ext:string;
begin
 // eine Stringlist als Zwischenspeicher
 stringlist:=Tstringlist.Create;
 stringlist.Add(self.Prof_Titel);
 for j:=low(self.koordinaten) to high(self.koordinaten) do
  begin
   // eine Zeile mit X- und Y-Koordinate für einen Punkt zusammenbauen
   zeile:=FormatFloat('0.00000000',self.koordinaten[j].x)+' '+
          FormatFloat('0.00000000',self.koordinaten[j].y);
   // je nach Spracheinstellung wird als Dzimalseparator ein '.' oder ','
   // verwendet
   // das Selig-Format muß einen '.'  als Dezimaltrenner haben
   // also eventuelle ',' durch einen '.' ersetzen
   zeile:=zeile.Replace(',','.');
   // Zeile an Stringlist anfügen
   stringlist.Add(zeile);
  end;

 // nachschauen , ob bei filename eine passende Dateierweiterung vorhanden ist
 ext:=uppercase(extractfileext(filename));
 if ext<>'.DAT'
  then
   begin
    // wenn ext leer
    if ext=''
     then filename:=filename+'.DAT'
     // wenn was anderes als '.DAT'drinn steht
     else
      begin
       filename:=copy(filename,1,pos('.',filename)-1);
       filename:=filename+'.DAT';
      end;
   end;

 // Stringlist abspeichern
 stringlist.SaveToFile(filename);

 // Stringlist wird nicht mehr benötigt , also killen (Speicher freigeben)
 stringlist.Free;
end;

// das Profil im Wiechert-Koo-Format speichern
procedure TProf.save_wiechert_koo_to_file(filename: string);
var stringlist:Tstringlist;
    j:integer; // Zeilenzähler
    zeile:string;
    ext:string;
begin
 // eine Stringlist als Zwischenspeicher
 stringlist:=Tstringlist.Create;
 // Titel und Anzahl der Koordinaten eintragen
 stringlist.Add(self.Prof_Titel+' , '+inttostr(high(self.koordinaten)));
 for j:=low(self.koordinaten) to high(self.koordinaten) do
  begin
   // eine Zeile mit X- und Y-Koordinate für einen Punkt zusammenbauen
   // Wiechers benutz eine Koordinatennormierung von 100 , also
   // alle Werte *100 nehmen
   zeile:=FormatFloat('0.00000',self.koordinaten[j].x*100)+'    '+
          FormatFloat('0.00000',self.koordinaten[j].y*100);
   // je nach Spracheinstellung wird als Dzimalseparator ein '.' oder ','
   // verwendet
   // das Wiechers-Format muß einen '.'  als Dezimaltrenner haben
   // also eventuelle ',' durch einen '.' ersetzen
   zeile:=zeile.Replace(',','.');
   // Zeile an Stringlist anfügen
   stringlist.Add(zeile);
  end;

 // nachschauen , ob bei filename eine passende Dateierweiterung vorhanden ist
 ext:=uppercase(extractfileext(filename));
 if ext<>'.KOO'
  then
   begin
    // wenn ext leer
    if ext=''
     then filename:=filename+'.KOO'
     // wenn was anderes als '.KOO'drinn steht
     else
      begin
       filename:=copy(filename,1,pos('.',filename)-1);
       filename:=filename+'.KOO';
      end;
   end;

 // Stringlist abspeichern
 stringlist.SaveToFile(filename);

 // Stringlist wird nicht mehr benötigt , also killen (Speicher freigeben)
 stringlist.Free;
end;



// Überträgt die Profilkoordinaten an das Chart
procedure TProf.send_to_Chart(chart: tchart);
var LineSeries:TLineSeries;
    i:integer;
begin
 if chart=nil then exit;
 if self.count=0 then exit;

 // Das Koordinaten-doublearray an Chart übertragen
 // Alten Inhalt killen
 chart.ClearSeries;
 // auf Proportial setzten , sonst wird das Profil verzerrt
 chart.Proportional:=true;
 // neue Lineserie erzeugen
 LineSeries:=TLineSeries.Create(chart);
 // Farbe der Serie
 LineSeries.SeriesColor:=clred;
 // Punkte anzeigen
 LineSeries.ShowPoints:=true;
 // Runde Punkte , Füllfarbe rot
 LineSeries.Pointer.Style:=TSeriesPointerStyle(psCircle);
 LineSeries.Pointer.Brush.Color:=clred;
 // Linienbreite
 LineSeries.LinePen.Width:=2;
 // Größe der Punkte
 LineSeries.Pointer.HorizSize:=3;
 LineSeries.Pointer.VertSize:=3;

 // Profil-Daten i die LineSerie übertragen
 for i:=low(self.koordinaten) to high(self.koordinaten) do
  begin
   LineSeries.AddXY(self.koordinaten[i].x,self.koordinaten[i].y);
  end;
 chart.AddSeries(LineSeries);
end;


end.

